/**
 * Boids v4, 
 * Nicolas Liautaud 2009
 * <br/>
 * Boids flock simulation based on Craig Reynold's algorythm.
 * <br/>
 * Use BoidFlock, Obj and FpsCounter class
 */

ArrayList foodList, obstList;
int foodNbr, obstNbr, boidsNbr;
boolean displayAura, displayPerception;
boolean changeObstAura, changeFoodAura;
float obstAuraFactor, foodAuraFactor;
BoidsFlock boids;
FpsCounter fps;

void setup() 
{ 
  // canvas
  size(600, 600);
  frameRate(30);
  smooth();

  // initialize obstacles
  obstList = new ArrayList();
  obstNbr  = 10;
  obstAuraFactor = 3;
  // initialize food
  foodList = new ArrayList();
  foodNbr  = 10;
  foodAuraFactor = 3;
  // initialize boids
  boids = new BoidsFlock();
  boidsNbr = 200;

  // initialize controls
  controlsInit();
  fps = new FpsCounter(width - 56, 42);

  // create foods
  for(int i=0; i < foodNbr; i++)
  {
    foodList.add(new Obj());
    Obj o = (Obj)foodList.get(i);
    o.aura = o.rad * foodAuraFactor;
  }
  // create obstacles
  for(int i=0; i < obstNbr; i++)
  {
    obstList.add(new Obj());
    Obj o = (Obj)obstList.get(i);
    o.aura = o.rad * obstAuraFactor;
  }
  // create boids
  for(int i=0; i < boidsNbr; i++)
  {
    boids.add();
  }
}

void draw() 
{
  background(255);

  // draw details (controls manipulation)
  if(displayPerception) boids.displayPerception();
  if(displayAura) boids.displayAura();
  if(changeObstAura) changeObjAura(obstList, obstAuraFactor, 100, 100, 100);
  if(changeFoodAura) changeObjAura(foodList, foodAuraFactor, 100, 255, 100);

  // draw food
  for(int i=0; i<foodList.size(); i++)
  {
    Obj f = (Obj) foodList.get(i);
    f.display(120, 255, 120);
  }
  // draw obstacles
  for(int i=0; i<obstList.size(); i++)
  {
    Obj o = (Obj) obstList.get(i);
    o.display();
  }

  // draw boids and zone border
  boids.display();
  boids.zone.border(150);

  // hide control zone background and draw fps
  fill(255); 
  noStroke();
  rect(0, 0, width, 50);
  fps.display();

  // compute
  boids.update();

} 


void mousePressed()
{
  // check if clic is in sim zone
  if(boids.zone.isIn(mouseX, mouseY))
  {
    // add food on left clic
    if(mouseButton == LEFT)
    {
      foodList.add(new Obj(mouseX, mouseY));
    }
    // add obst on right clic
    else if(mouseButton == RIGHT)
    {
      obstList.add(new Obj(mouseX, mouseY));
    }
    // delete things with middle clic
    else if(mouseButton == CENTER)
    {
      // delete food if mouse is on it
      for(int i=0; i<foodList.size(); i++){
        Obj o = (Obj) foodList.get(i);
        if(dist(o.pos.x, o.pos.y, mouseX, mouseY) < o.rad)
        {
          foodList.remove(o);
          break;
        }
      }
      // delete obstacle if mouse is on it
      for(int i=0; i<obstList.size(); i++){
        Obj o = (Obj) obstList.get(i);
        if(dist(o.pos.x, o.pos.y, mouseX, mouseY) < o.rad)
        {
          obstList.remove(o);
          break;
        }
      }
    }
  }
}

// stop displaying details on mouse releasing
void mouseReleased()
{
  displayAura = false;
  displayPerception = false;
  changeFoodAura = false;
  changeObstAura = false;
}

// reset on SPACE press
void keyPressed()
{
  if(key == ' ') setup();
}

// change aura value and display it (controls manipulation)
void changeObjAura(ArrayList l, float f, int r, int g, int b)
{
  fill(r, g, b, 20); 
  stroke(r, g, b, 50);
  for(int i=0; i < l.size(); i++)
  {
    Obj o = (Obj) l.get(i);
    o.aura = o.rad * f;
    ellipse(o.pos.x,  o.pos.y, o.aura * 2, o.aura * 2);
  }
}




